using System;
using System.Collections.Generic;
using System.Text;
using JetBrains.ReSharper.Daemon;
using JetBrains.ReSharper.Daemon.CSharp.Stages;
using JetBrains.ReSharper.Daemon.Impl;
using JetBrains.ReSharper.Psi;
using JetBrains.ReSharper.Psi.CSharp;

namespace ExitPointsHighlighter
{
    class ExitPointsHighligherStageProcess : CSharpDaemonStageProcessBase 
    {
        public ExitPointsHighligherStageProcess(IDaemonProcess process):base(process)
        {
            
        }

        public override void ProcessFile(ICSharpFile file)
        {
            base.ProcessElement(file);
            base.FullyRehighlighted = true;
        }

        public override void ProcessAfterInterior(JetBrains.ReSharper.Psi.IElement element)
        {
            if (IsExitPoint(element))
            {
                base.AddHighlighting(new IdentifierHighlighting(element.ToTreeNode().FirstChild.GetDocumentRange(), "ReSharper Event Identifier"));
                return;
            }
            if (IsFlowKeyWord(element))
            {
                base.AddHighlighting(new IdentifierHighlighting(element.ToTreeNode().FirstChild.GetDocumentRange(), "ReSharper Constant Identifier"));
                return;
            }
                
        }

        private bool IsExitPoint(IElement element)
        {
            if (element is IReturnStatementNode)
                return true;
            if (element is IThrowStatementNode)
                return true;
           
            return false;
        }

        private bool IsFlowKeyWord(IElement element)
        {
            if (element is IBreakStatementNode)
                return true;
            if (element is IContinueStatementNode)
                return true;
            if (element is IGotoStatementNode)
                return true;

            return false;
        }

        
    }
}
